@@ -26,6 +26,8 @@
 #include "qgsprocessing.h"
 #include "qgsfeaturesink.h"
 #include "qgsfeaturesource.h"
+#include "qgsproxyfeaturesink.h"
+#include "qgsremappingproxyfeaturesink.h"
 
 class QgsMeshLayer;
 class QgsProject;
@@ -34,6 +36,7 @@ class QgsMapLayerStore;
 class QgsProcessingFeedback;
 class QgsProcessingFeatureSource;
 class QgsProcessingAlgorithm;
+class QgsVectorTileLayer;
 
 #include <QString>
 #include <QVariant>
@@ -105,6 +108,27 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static QList< QgsMapLayer * > compatibleLayers( QgsProject *project, bool sort = true );
 
+    /**
+     * Encodes a provider key and layer \a uri to a single string, for use with
+     * decodeProviderKeyAndUri()
+     *
+     * \since QGIS 3.14
+     */
+    static QString encodeProviderKeyAndUri( const QString &providerKey, const QString &uri );
+
+    /**
+     * Decodes a provider key and layer \a uri from an encoded string, for use with
+     * encodeProviderKeyAndUri()
+     *
+     * \param string encoded string, as returned by encodeProviderKeyAndUri()
+     * \param providerKey ID key for corresponding data provider
+     * \param uri decoded layer uri
+     * \returns TRUE if \a string was successfully decoded
+     *
+     * \since QGIS 3.14
+     */
+    static bool decodeProviderKeyAndUri( const QString &string, QString &providerKey SIP_OUT, QString &uri SIP_OUT );
+
     /**
      * Layer type hints.
      * \since QGIS 3.4
@@ -114,7 +138,7 @@ class CORE_EXPORT QgsProcessingUtils
       UnknownType, //!< Unknown layer type
       Vector, //!< Vector layer type
       Raster, //!< Raster layer type
-      Mesh, //!< Mesh layer type  \since QGIS 3.6
+      Mesh, //!< Mesh layer type, since QGIS 3.6
     };
 
     /**
@@ -144,6 +168,16 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static QgsProcessingFeatureSource *variantToSource( const QVariant &value, QgsProcessingContext &context, const QVariant &fallbackValue = QVariant() ) SIP_FACTORY;
 
+    /**
+     * Converts a variant \a value to a coordinate reference system.
+     *
+     * The optional \a fallbackValue can be used to specify a "default" value which is used
+     * if \a value cannot be successfully converted to a CRS.
+     *
+     * \since QGIS 3.12
+     */
+    static QgsCoordinateReferenceSystem variantToCrs( const QVariant &value, QgsProcessingContext &context, const QVariant &fallbackValue = QVariant() );
+
     /**
      * Normalizes a layer \a source string for safe comparison across different
      * operating system environments.
@@ -189,7 +223,10 @@ class CORE_EXPORT QgsProcessingUtils
         QgsWkbTypes::Type geometryType,
         const QgsCoordinateReferenceSystem &crs,
         const QVariantMap &createOptions = QVariantMap(),
-        QgsFeatureSink::SinkFlags sinkFlags = nullptr ) SIP_FACTORY;
+        const QStringList &datasourceOptions = QStringList(),
+        const QStringList &layerOptions = QStringList(),
+        QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(),
+        QgsRemappingSinkDefinition *remappingDefinition = nullptr ) SIP_FACTORY;
 #endif
 
     /**
@@ -269,6 +306,8 @@ class CORE_EXPORT QgsProcessingUtils
      * The \a preferredFormat argument is used to specify to desired file extension to use when a temporary
      * layer export is required. This defaults to shapefiles.
      *
+     * The \a featureLimit argument can be used to specify a limit on the number of features read from the layer.
+     *
      * When an algorithm is capable of handling multi-layer input files (such as Geopackage), it is preferable
      * to use convertToCompatibleFormatAndLayerName() which may avoid conversion in more situations.
      *
@@ -280,7 +319,7 @@ class CORE_EXPORT QgsProcessingUtils
         const QStringList &compatibleFormats,
         const QString &preferredFormat,
         QgsProcessingContext &context,
-        QgsProcessingFeedback *feedback );
+        QgsProcessingFeedback *feedback, long long featureLimit = -1 );
 
     /**
      * Converts a source vector \a layer to a file path and layer name of a vector layer of compatible format.
@@ -291,6 +330,8 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * \a compatibleFormats should consist entirely of lowercase file extensions, e.g. 'shp'.
      *
+     * The \a featureLimit argument can be used to specify a limit on the number of features read from the layer.
+     *
      * The \a preferredFormat argument is used to specify to desired file extension to use when a temporary
      * layer export is required. This defaults to shapefiles.
      *
@@ -306,6 +347,7 @@ class CORE_EXPORT QgsProcessingUtils
      * \param context processing context
      * \param feedback feedback object
      * \param layerName will be set to the target layer name for multi-layer sources (e.g. Geopackage)
+     * \param featureLimit can be used to place a limit on the maximum number of features read from the layer
      *
      * \returns path to source layer, or nearly converted compatible layer
      *
@@ -319,7 +361,7 @@ class CORE_EXPORT QgsProcessingUtils
         const QString &preferredFormat,
         QgsProcessingContext &context,
         QgsProcessingFeedback *feedback,
-        QString &layerName SIP_OUT );
+        QString &layerName SIP_OUT, long long featureLimit = -1 );
 
     /**
      * Combines two field lists, avoiding duplicate field names (in a case-insensitive manner).
@@ -373,6 +415,7 @@ class CORE_EXPORT QgsProcessingUtils
   private:
     static bool canUseLayer( const QgsRasterLayer *layer );
     static bool canUseLayer( const QgsMeshLayer *layer );
+    static bool canUseLayer( const QgsVectorTileLayer *layer );
     static bool canUseLayer( const QgsVectorLayer *layer,
                              const QList< int > &sourceTypes = QList< int >() );
 
@@ -441,8 +484,12 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
      * Ownership of \a originalSource is dictated by \a ownsOriginalSource. If \a ownsOriginalSource is FALSE,
      * ownership is not transferred, and callers must ensure that \a originalSource exists for the lifetime of this object.
      * If \a ownsOriginalSource is TRUE, then this object will take ownership of \a originalSource.
+     *
+     * If \a featureLimit is set to a value > 0, then a limit is placed on the maximum number of features which will be
+     * read from the source.
      */
-    QgsProcessingFeatureSource( QgsFeatureSource *originalSource, const QgsProcessingContext &context, bool ownsOriginalSource = false );
+    QgsProcessingFeatureSource( QgsFeatureSource *originalSource, const QgsProcessingContext &context, bool ownsOriginalSource = false,
+                                long long featureLimit = -1 );
 
     ~QgsProcessingFeatureSource() override;
 
@@ -466,12 +513,20 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     QVariant maximumValue( int fieldIndex ) const override;
     QgsRectangle sourceExtent() const override;
     QgsFeatureIds allFeatureIds() const override;
+    SpatialIndexPresence hasSpatialIndex() const override;
 
     /**
      * Returns an expression context scope suitable for this source.
      */
     QgsExpressionContextScope *createExpressionContextScope() const SIP_FACTORY;
 
+    /**
+     * Overrides the default geometry check method for the source.
+     *
+     * \since QGIS 3.14
+     */
+    void setInvalidGeometryCheck( QgsFeatureRequest::InvalidGeometryCheck method );
+
   private:
 
     QgsFeatureSource *mSource = nullptr;
@@ -480,6 +535,11 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallback;
     std::function< void( const QgsFeature & ) > mTransformErrorCallback;
 
+    std::function< void( const QgsFeature & ) > mInvalidGeometryCallbackSkip;
+    std::function< void( const QgsFeature & ) > mInvalidGeometryCallbackAbort;
+
+    long long mFeatureLimit = -1;
+
 };
 
 #ifndef SIP_RUN
@@ -511,9 +571,9 @@ class CORE_EXPORT QgsProcessingFeatureSink : public QgsProxyFeatureSink
      */
     QgsProcessingFeatureSink( QgsFeatureSink *originalSink, const QString &sinkName, QgsProcessingContext &context, bool ownsOriginalSink = false );
     ~QgsProcessingFeatureSink() override;
-    bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = nullptr ) override;
-    bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = nullptr ) override;
-    bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = nullptr ) override;
+    bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
 
   private:
 
