@@ -33,13 +33,14 @@
 
 #include "qgscolorrampshader.h"
 #include "qgsdataprovider.h"
-#include "qgsfields.h"
 #include "qgsraster.h"
+#include "qgsfields.h"
 #include "qgsrasterinterface.h"
 #include "qgsrasterpyramid.h"
 #include "qgsrasterrange.h"
 #include "qgsrectangle.h"
 #include "qgsrasteriterator.h"
+#include "qgsrasterdataprovidertemporalcapabilities.h"
 
 class QImage;
 class QByteArray;
@@ -62,14 +63,16 @@ class CORE_EXPORT QgsImageFetcher : public QObject
 
     /**
      * Starts the image download
-     * \note Make sure to connect to "finish" and "error" before starting */
+     * \note Make sure to connect to "finish" and "error" before starting
+    */
     virtual void start() = 0;
 
   signals:
 
     /**
      * Emitted when the download completes
-     *  \param legend The downloaded legend image */
+     *  \param legend The downloaded legend image
+    */
     void finish( const QImage &legend );
     //! Emitted to report progress
     void progress( qint64 received, qint64 total );
@@ -97,6 +100,8 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
       NoProviderCapabilities = 0,       //!< Provider has no capabilities
       ReadLayerMetadata = 1 << 1, //!< Provider can read layer metadata from data store. Since QGIS 3.0. See QgsDataProvider::layerMetadata()
       WriteLayerMetadata = 1 << 2, //!< Provider can write layer metadata to the data store. Since QGIS 3.0. See QgsDataProvider::writeLayerMetadata()
+      ProviderHintBenefitsFromResampling = 1 << 3, //!< Provider benefits from resampling and should apply user default resampling settings (since QGIS 3.10)
+      ProviderHintCanPerformProviderResampling = 1 << 4 //!< Provider can perform resampling (to be opposed to post rendering resampling) (since QGIS 3.16)
     };
 
     //! Provider capabilities
@@ -110,11 +115,13 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * The \a uri argument gives a provider-specific uri indicating the underlying data
      * source and it's parameters.
      *
-     * The \a options argument specifies generic provider options.
+     * The \a options argument specifies generic provider options and since QGIS 3.16 creation flags are specified within the \a flags value.
      */
-    QgsRasterDataProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options = QgsDataProvider::ProviderOptions() );
+    QgsRasterDataProvider( const QString &uri,
+                           const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions(),
+                           QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
-    QgsRasterInterface *clone() const override = 0;
+    QgsRasterDataProvider *clone() const override = 0;
 
     /**
      * Returns flags containing the supported capabilities of the data provider.
@@ -123,13 +130,20 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     virtual QgsRasterDataProvider::ProviderCapabilities providerCapabilities() const;
 
     /* It makes no sense to set input on provider */
-    bool setInput( QgsRasterInterface *input ) override { Q_UNUSED( input ); return false; }
+    bool setInput( QgsRasterInterface *input ) override { Q_UNUSED( input ) return false; }
 
     QgsRectangle extent() const override = 0;
 
     //! Returns data type for the band specified by number
     Qgis::DataType dataType( int bandNo ) const override = 0;
 
+    /**
+     * Returns the fields of the raster layer for data providers that expose them,
+     * the default implementation returns an empty list.
+     * \since QGIS 3.14
+     */
+    virtual QgsFields fields() const { return QgsFields(); };
+
     /**
      * Returns source data type for the band specified by number,
      *  source data type may be shorter than dataType
@@ -139,7 +153,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     //! Returns data type for the band specified by number
     virtual int colorInterpretation( int bandNo ) const
     {
-      Q_UNUSED( bandNo );
+      Q_UNUSED( bandNo )
       return QgsRaster::UndefinedColorInterpretation;
     }
 
@@ -215,20 +229,20 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * Read band scale for raster value
      * \since QGIS 2.3
      */
-    virtual double bandScale( int bandNo ) const { Q_UNUSED( bandNo ); return 1.0; }
+    virtual double bandScale( int bandNo ) const { Q_UNUSED( bandNo ) return 1.0; }
 
     /**
      * Read band offset for raster value
      * \since QGIS 2.3
      */
-    virtual double bandOffset( int bandNo ) const { Q_UNUSED( bandNo ); return 0.0; }
+    virtual double bandOffset( int bandNo ) const { Q_UNUSED( bandNo ) return 0.0; }
 
     // TODO: remove or make protected all readBlock working with void*
 
     //! Read block of data using given extent and size.
-    QgsRasterBlock *block( int bandNo, const QgsRectangle &boundingBox, int width, int height, QgsRasterBlockFeedback *feedback = nullptr ) override;
+    QgsRasterBlock *block( int bandNo, const QgsRectangle &boundingBox, int width, int height, QgsRasterBlockFeedback *feedback = nullptr ) override SIP_FACTORY;
 
-    //! Returns true if source band has no data value
+    //! Returns TRUE if source band has no data value
     virtual bool sourceHasNoDataValue( int bandNo ) const { return mSrcHasNoDataValue.value( bandNo - 1 ); }
 
     //! Returns the source nodata value usage
@@ -246,16 +260,20 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     virtual QgsRasterRangeList userNoDataValues( int bandNo ) const { return mUserNoDataValue.value( bandNo - 1 ); }
 
     virtual QList<QgsColorRampShader::ColorRampItem> colorTable( int bandNo ) const
-    { Q_UNUSED( bandNo ); return QList<QgsColorRampShader::ColorRampItem>(); }
+    { Q_UNUSED( bandNo ) return QList<QgsColorRampShader::ColorRampItem>(); }
 
     /**
      * \brief Returns the sublayers of this layer - useful for providers that manage
-     *  their own layers, such as WMS */
+     *  their own layers, such as WMS
+    */
     QStringList subLayers() const override
     {
       return QStringList();
     }
 
+    QgsRasterDataProviderTemporalCapabilities *temporalCapabilities() override;
+    const QgsRasterDataProviderTemporalCapabilities *temporalCapabilities() const override SIP_SKIP;
+
     //! \brief Returns whether the provider supplies a legend graphic
     virtual bool supportsLegendGraphic() const { return false; }
 
@@ -272,9 +290,9 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual QImage getLegendGraphic( double scale = 0, bool forceRefresh = false, const QgsRectangle *visibleExtent = nullptr ) SIP_SKIP
     {
-      Q_UNUSED( scale );
-      Q_UNUSED( forceRefresh );
-      Q_UNUSED( visibleExtent );
+      Q_UNUSED( scale )
+      Q_UNUSED( forceRefresh )
+      Q_UNUSED( visibleExtent )
       return QImage();
     }
 
@@ -284,7 +302,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * \param mapSettings map settings for legend providers supporting
      *                    contextual legends.
      *
-     * \returns a download handler or null if the provider does not support
+     * \returns a download handler or NULLPTR if the provider does not support
      *         legend at all. Ownership of the returned object is transferred
      *         to caller.
      *
@@ -293,7 +311,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual QgsImageFetcher *getLegendGraphicFetcher( const QgsMapSettings *mapSettings ) SIP_FACTORY
     {
-      Q_UNUSED( mapSettings );
+      Q_UNUSED( mapSettings )
       return nullptr;
     }
 
@@ -304,11 +322,11 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
                                    const QStringList &configOptions = QStringList(),
                                    QgsRasterBlockFeedback *feedback = nullptr )
     {
-      Q_UNUSED( pyramidList );
-      Q_UNUSED( resamplingMethod );
-      Q_UNUSED( format );
-      Q_UNUSED( configOptions );
-      Q_UNUSED( feedback );
+      Q_UNUSED( pyramidList )
+      Q_UNUSED( resamplingMethod )
+      Q_UNUSED( format )
+      Q_UNUSED( configOptions )
+      Q_UNUSED( feedback )
       return QStringLiteral( "FAILED_NOT_SUPPORTED" );
     }
 
@@ -321,9 +339,9 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * list.
      */
     virtual QList<QgsRasterPyramid> buildPyramidList( QList<int> overviewList = QList<int>() ) // clazy:exclude=function-args-by-ref
-    { Q_UNUSED( overviewList ); return QList<QgsRasterPyramid>(); }
+    { Q_UNUSED( overviewList ) return QList<QgsRasterPyramid>(); }
 
-    //! \brief Returns true if raster has at least one populated histogram.
+    //! \brief Returns TRUE if raster has at least one populated histogram.
     bool hasPyramids();
 
     /**
@@ -350,15 +368,32 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * \return QgsRaster::IdentifyFormatValue: map of values for each band, keys are band numbers
      *         (from 1).
      *         QgsRaster::IdentifyFormatFeature: map of QgsRasterFeatureList for each sublayer
-     *         (WMS) - TODO: it is not consistent with QgsRaster::IdentifyFormatValue.
      *         QgsRaster::IdentifyFormatHtml: map of HTML strings for each sublayer (WMS).
-     *         Empty if failed or there are no results (TODO: better error reporting).
+     *         Empty if failed or there are no results.
      * \note The arbitraryness of the returned document is enforced by WMS standards
      *       up to at least v1.3.0
+     * \see sample(), which is much more efficient for simple "value at point" queries.
      */
-    //virtual QMap<int, QVariant> identify( const QgsPointXY & point, QgsRaster::IdentifyFormat format, const QgsRectangle &extent = QgsRectangle(), int width = 0, int height = 0 );
     virtual QgsRasterIdentifyResult identify( const QgsPointXY &point, QgsRaster::IdentifyFormat format, const QgsRectangle &boundingBox = QgsRectangle(), int width = 0, int height = 0, int dpi = 96 );
 
+    /**
+     * Samples a raster value from the specified \a band found at the \a point position. The context
+     * parameters \a boundingBox, \a width and \a height are important to identify
+     * on the same zoom level as a displayed map and to do effective
+     * caching (WCS). If context params are not specified the highest
+     * resolution is used.
+     *
+     * If \a ok is specified and the point is outside data source extent, or an invalid
+     * band number was specified, then \a ok will be set to FALSE. In this case the function will return
+     * a NaN value.
+     *
+     * \see identify(), which is much more flexible but considerably less efficient.
+     * \since QGIS 3.4
+     */
+    virtual double sample( const QgsPointXY &point, int band,
+                           bool *ok SIP_OUT = nullptr,
+                           const QgsRectangle &boundingBox = QgsRectangle(), int width = 0, int height = 0, int dpi = 96 );
+
     /**
      * \brief Returns the caption error text for the last error in this provider
      *
@@ -406,25 +441,26 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     /**
      * Turns on/off editing mode of the provider. When in editing mode, it is possible
      * to overwrite data of the provider using writeBlock() calls.
-     * \returns true if the switch to/from editing mode was successful
+     * \returns TRUE if the switch to/from editing mode was successful
      * \note Only some providers support editing mode and even those may fail to turn
      * the underlying data source into editing mode, so it is necessary to check the return
      * value whether the operation was successful.
      * \see isEditable(), writeBlock()
      * \since QGIS 3.0
      */
-    virtual bool setEditable( bool enabled ) { Q_UNUSED( enabled ); return false; }
+    virtual bool setEditable( bool enabled ) { Q_UNUSED( enabled ) return false; }
+
+    // TODO: add data type (may be different from band type)
 
     //! Writes into the provider datasource
-    // TODO: add data type (may be different from band type)
     virtual bool write( void *data, int band, int width, int height, int xOffset, int yOffset )
     {
-      Q_UNUSED( data );
-      Q_UNUSED( band );
-      Q_UNUSED( width );
-      Q_UNUSED( height );
-      Q_UNUSED( xOffset );
-      Q_UNUSED( yOffset );
+      Q_UNUSED( data )
+      Q_UNUSED( band )
+      Q_UNUSED( width )
+      Q_UNUSED( height )
+      Q_UNUSED( xOffset )
+      Q_UNUSED( yOffset )
       return false;
     }
 
@@ -441,7 +477,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * Writing is supported only by some data providers. Provider has to be in editing mode
      * in order to allow write operations.
      * \see isEditable(), setEditable()
-     * \returns true on success
+     * \returns TRUE on success
      * \since QGIS 3.0
      */
     bool writeBlock( QgsRasterBlock *block, int band, int xOffset = 0, int yOffset = 0 );
@@ -460,7 +496,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      *  \param bandNo band number
      *  \param noDataValue no data value
      */
-    virtual bool setNoDataValue( int bandNo, double noDataValue ) { Q_UNUSED( bandNo ); Q_UNUSED( noDataValue ); return false; }
+    virtual bool setNoDataValue( int bandNo, double noDataValue ) { Q_UNUSED( bandNo ) Q_UNUSED( noDataValue ); return false; }
 
     //! Remove dataset
     virtual bool remove() { return false; }
@@ -477,7 +513,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * \note see also validateCreationOptionsFormat() in gdal provider for validating options based on format only
      */
     virtual QString validateCreationOptions( const QStringList &createOptions, const QString &format )
-    { Q_UNUSED( createOptions ); Q_UNUSED( format ); return QString(); }
+    { Q_UNUSED( createOptions ) Q_UNUSED( format ); return QString(); }
 
     /**
      * Validates pyramid creation options for a specific dataset and destination format
@@ -485,7 +521,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual QString validatePyramidsConfigOptions( QgsRaster::RasterPyramidsFormat pyramidsFormat,
         const QStringList &configOptions, const QString &fileFormat )
-    { Q_UNUSED( pyramidsFormat ); Q_UNUSED( configOptions ); Q_UNUSED( fileFormat ); return QString(); }
+    { Q_UNUSED( pyramidsFormat ) Q_UNUSED( configOptions ); Q_UNUSED( fileFormat ); return QString(); }
 
     static QString identifyFormatName( QgsRaster::IdentifyFormat format );
     static QgsRaster::IdentifyFormat identifyFormatFromName( const QString &formatName );
@@ -506,6 +542,128 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual int stepHeight() const { return QgsRasterIterator::DEFAULT_MAXIMUM_TILE_HEIGHT; }
 
+    /**
+     * Returns a list of native resolutions if available, i.e. map units per pixel at which the raster source
+     * was originally created.
+     *
+     * Resolutions are calculated in the provider's crs().
+     *
+     * \since QGIS 3.8.0
+     */
+    virtual QList< double > nativeResolutions() const;
+
+    /**
+     * Returns TRUE if the extents reported by the data provider are not reliable
+     * and it's possible that there is renderable content outside of these extents.
+     *
+     * \since QGIS 3.10.0
+     */
+    virtual bool ignoreExtents() const;
+
+    /**
+     * Types of transformation in transformCoordinates() function.
+     * \since QGIS 3.14
+     */
+    enum TransformType
+    {
+      TransformImageToLayer,  //!< Transforms image coordinates to layer (georeferenced) coordinates
+      TransformLayerToImage,  //!< Transforms layer (georeferenced) coordinates to image coordinates
+    };
+
+    /**
+     * Transforms coordinates between source image coordinate space [0..width]x[0..height] and
+     * layer coordinate space (georeferenced coordinates). Often this transformation is a simple
+     * 2D affine transformation (offset and scaling), but rasters with different georeferencing
+     * methods like GCPs (ground control points) or RPCs (rational polynomial coefficients) may
+     * require a more complex transform.
+     *
+     * If the transform fails (input coordinates are outside of the valid range or data provider
+     * does not support this functionality), an empty point is returned.
+     *
+     * \since QGIS 3.14
+     */
+    virtual QgsPoint transformCoordinates( const QgsPoint &point, TransformType type );
+
+
+    /**
+     * Enable or disable provider-level resampling.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool enableProviderResampling( bool enable ) { Q_UNUSED( enable ); return false; }
+
+    /**
+     * Returns whether provider-level resampling is enabled.
+     *
+     * \note Resampling is effective only if zoomedInResamplingMethod() and/or
+     * zoomedOutResamplingMethod() return non-nearest resampling.
+     *
+     * \see zoomedInResamplingMethod()
+     * \see zoomedOutResamplingMethod()
+     * \see maxOversampling()
+     *
+     * \since QGIS 3.16
+     */
+    bool isProviderResamplingEnabled() const { return mProviderResamplingEnabled; }
+
+    /**
+     * Resampling method for provider-level resampling.
+     * \since QGIS 3.16
+     */
+    enum class ResamplingMethod
+    {
+      Nearest,      //!< Nearest-neighbour resamplikng
+      Bilinear,     //!< Bilinear resamplikng
+      Cubic,     //!< Bicubic resamplikng
+    };
+
+    /**
+     * Set resampling method to apply for zoomed-in operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setZoomedInResamplingMethod( ResamplingMethod method ) { Q_UNUSED( method ); return false; }
+
+    /**
+     * Returns resampling method for zoomed-in operations.
+     * \since QGIS 3.16
+     */
+    ResamplingMethod zoomedInResamplingMethod() const { return mZoomedInResamplingMethod; }
+
+    /**
+     * Set resampling method to apply for zoomed-out operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setZoomedOutResamplingMethod( ResamplingMethod method ) { Q_UNUSED( method ); return false; }
+
+    /**
+     * Returns resampling method for zoomed-out operations.
+     * \since QGIS 3.16
+     */
+    ResamplingMethod zoomedOutResamplingMethod() const { return mZoomedOutResamplingMethod; }
+
+    /**
+     * Sets maximum oversampling factor for zoomed-out operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setMaxOversampling( double factor ) { Q_UNUSED( factor ); return false; }
+
+    /**
+     * Returns maximum oversampling factor for zoomed-out operations.
+     * \since QGIS 3.16
+     */
+    double maxOversampling() const { return mMaxOversampling; }
+
+    void readXml( const QDomElement &filterElem ) override;
+
+    void writeXml( QDomDocument &doc, QDomElement &parentElem ) const override;
+
   signals:
 
     /**
@@ -517,20 +675,22 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
   protected:
 
     /**
-     * Read block of data
+     * Reads a block of raster data into \a data.
+     * \returns TRUE if the block was successfully read, or FALSE if an error occurred and the block could not be read.
      * \note not available in Python bindings
      */
-    virtual void readBlock( int bandNo, int xBlock, int yBlock, void *data ) SIP_SKIP
-    { Q_UNUSED( bandNo ); Q_UNUSED( xBlock ); Q_UNUSED( yBlock ); Q_UNUSED( data ); }
+    virtual bool readBlock( int bandNo, int xBlock, int yBlock, void *data ) SIP_SKIP
+    { Q_UNUSED( bandNo ) Q_UNUSED( xBlock ); Q_UNUSED( yBlock ); Q_UNUSED( data ); return false; }
 
     /**
-     * Read block of data using give extent and size
+     * Reads a block of raster data into \a data, using the given extent and size.
+     * \returns TRUE if the block was successfully read, or FALSE if an error occurred and the block could not be read.
      * \note not available in Python bindings
      */
-    virtual void readBlock( int bandNo, QgsRectangle  const &viewExtent, int width, int height, void *data, QgsRasterBlockFeedback *feedback = nullptr ) SIP_SKIP
-    { Q_UNUSED( bandNo ); Q_UNUSED( viewExtent ); Q_UNUSED( width ); Q_UNUSED( height ); Q_UNUSED( data ); Q_UNUSED( feedback ); }
+    virtual bool readBlock( int bandNo, QgsRectangle  const &viewExtent, int width, int height, void *data, QgsRasterBlockFeedback *feedback = nullptr ) SIP_SKIP
+    { Q_UNUSED( bandNo ) Q_UNUSED( viewExtent ); Q_UNUSED( width ); Q_UNUSED( height ); Q_UNUSED( data ); Q_UNUSED( feedback ); return false; }
 
-    //! Returns true if user no data contains value
+    //! Returns TRUE if user no data contains value
     bool userNoDataValuesContains( int bandNo, double value ) const;
 
     //! Copy member variables from other raster data provider. Useful for implementation of clone() method in subclasses
@@ -538,12 +698,14 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Dots per inch. Extended WMS (e.g. QGIS mapserver) support DPI dependent output and therefore
-    are suited for printing. A value of -1 means it has not been set */
+    are suited for printing. A value of -1 means it has not been set
+    */
     int mDpi = -1;
 
     /**
      * Source no data value is available and is set to be used or internal no data
-     *  is available. Used internally only  */
+     *  is available. Used internally only
+    */
     //bool hasNoDataValue ( int bandNo );
 
     //! \brief Cell value representing original source no data. e.g. -9999, indexed from 0
@@ -555,16 +717,37 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     /**
      * \brief Use source nodata value. User can disable usage of source nodata
      *  value as nodata. It may happen that a value is wrongly given by GDAL
-     *  as nodata (e.g. 0) and it has to be treated as regular value. */
+     *  as nodata (e.g. 0) and it has to be treated as regular value.
+    */
     QList<bool> mUseSrcNoDataValue;
 
     /**
      * \brief List of lists of user defined additional no data values
-     *  for each band, indexed from 0 */
+     *  for each band, indexed from 0
+    */
     QList< QgsRasterRangeList > mUserNoDataValue;
 
     mutable QgsRectangle mExtent;
 
+    //! Whether provider resampling is enabled.
+    bool mProviderResamplingEnabled = false;
+
+    //! Resampling method for zoomed in pixel extraction
+    ResamplingMethod mZoomedInResamplingMethod = ResamplingMethod::Nearest;
+
+    //! Resampling method for zoomed out pixel extraction
+    ResamplingMethod mZoomedOutResamplingMethod = ResamplingMethod::Nearest;
+
+    //! Maximum boundary for oversampling (to avoid too much data traffic). Default: 2.0
+    double mMaxOversampling = 2.0;
+
+  private:
+
+    /**
+     * Data provider temporal properties
+     */
+    std::unique_ptr< QgsRasterDataProviderTemporalCapabilities > mTemporalCapabilities;
+
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsRasterDataProvider::ProviderCapabilities )
