@@ -20,8 +20,7 @@
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '9b176802e5512bc310765d8ebe4e44b5f4663453'
+
 
 #############################################################################
 #
@@ -97,6 +96,7 @@ Steve J. Fortune (1987) A Sweepline Algorithm for Voronoi Diagrams,
 Algorithmica 2, 153-174.
 """)
 
+
 #############################################################################
 #
 # For programmatic use two functions are available:
@@ -127,6 +127,7 @@ Algorithmica 2, 153-174.
 import math
 import sys
 import getopt
+
 TOLERANCE = 1e-9
 BIG_FLOAT = 1e38
 
@@ -140,11 +141,11 @@ class Context(object):
         self.debug = 0
         self.plot = 0
         self.triangulate = False
-        self.vertices = []    # list of vertex 2-tuples: (x,y)
-        self.lines = []    # equation of line 3-tuple (a b c), for the equation of the line a*x+b*y = c
-        self.edges = []    # edge 3-tuple: (line index, vertex 1 index, vertex 2 index)   if either vertex index is -1, the edge extends to infiinity
-        self.triangles = []    # 3-tuple of vertex indices
-        self.polygons = {}    # a dict of site:[edges] pairs
+        self.vertices = []  # list of vertex 2-tuples: (x,y)
+        self.lines = []  # equation of line 3-tuple (a b c), for the equation of the line a*x+b*y = c
+        self.edges = []  # edge 3-tuple: (line index, vertex 1 index, vertex 2 index)   if either vertex index is -1, the edge extends to infiinity
+        self.triangles = []  # 3-tuple of vertex indices
+        self.polygons = {}  # a dict of site:[edges] pairs
 
     def circle(self, x, y, rad):
         pass
@@ -224,146 +225,141 @@ class Context(object):
 
 
 def voronoi(siteList, context):
-    try:
-        edgeList = EdgeList(siteList.xmin, siteList.xmax, len(siteList))
-        priorityQ = PriorityQueue(siteList.ymin, siteList.ymax, len(siteList))
-        siteIter = siteList.iterator()
+    edgeList = EdgeList(siteList.xmin, siteList.xmax, len(siteList))
+    priorityQ = PriorityQueue(siteList.ymin, siteList.ymax, len(siteList))
+    siteIter = siteList.iterator()
 
-        bottomsite = next(siteIter)
-        context.outSite(bottomsite)
-        newsite = next(siteIter)
-        minpt = Site(-BIG_FLOAT, -BIG_FLOAT)
-        while True:
-            if not priorityQ.isEmpty():
-                minpt = priorityQ.getMinPt()
+    bottomsite = next(siteIter)
+    context.outSite(bottomsite)
+    newsite = next(siteIter)
+    minpt = Site(-BIG_FLOAT, -BIG_FLOAT)
+    while True:
+        if not priorityQ.isEmpty():
+            minpt = priorityQ.getMinPt()
 
-            if (newsite and (priorityQ.isEmpty() or cmp(newsite, minpt) < 0)):
-                # newsite is smallest -  this is a site event
-                context.outSite(newsite)
+        if (newsite and (priorityQ.isEmpty() or cmp(newsite, minpt) < 0)):
+            # newsite is smallest -  this is a site event
+            context.outSite(newsite)
 
-                # get first Halfedge to the LEFT and RIGHT of the new site
-                lbnd = edgeList.leftbnd(newsite)
-                rbnd = lbnd.right
+            # get first Halfedge to the LEFT and RIGHT of the new site
+            lbnd = edgeList.leftbnd(newsite)
+            rbnd = lbnd.right
 
-                # if this halfedge has no edge, bot = bottom site (whatever that is)
-                # create a new edge that bisects
-                bot = lbnd.rightreg(bottomsite)
-                edge = Edge.bisect(bot, newsite)
-                context.outBisector(edge)
+            # if this halfedge has no edge, bot = bottom site (whatever that is)
+            # create a new edge that bisects
+            bot = lbnd.rightreg(bottomsite)
+            edge = Edge.bisect(bot, newsite)
+            context.outBisector(edge)
 
-                # create a new Halfedge, setting its pm field to 0 and insert
-                # this new bisector edge between the left and right vectors in
-                # a linked list
-                bisector = Halfedge(edge, Edge.LE)
-                edgeList.insert(lbnd, bisector)
+            # create a new Halfedge, setting its pm field to 0 and insert
+            # this new bisector edge between the left and right vectors in
+            # a linked list
+            bisector = Halfedge(edge, Edge.LE)
+            edgeList.insert(lbnd, bisector)
 
-                # if the new bisector intersects with the left edge, remove
-                # the left edge's vertex, and put in the new one
-                p = lbnd.intersect(bisector)
-                if p is not None:
-                    priorityQ.delete(lbnd)
-                    priorityQ.insert(lbnd, p, newsite.distance(p))
+            # if the new bisector intersects with the left edge, remove
+            # the left edge's vertex, and put in the new one
+            p = lbnd.intersect(bisector)
+            if p is not None:
+                priorityQ.delete(lbnd)
+                priorityQ.insert(lbnd, p, newsite.distance(p))
 
-                # create a new Halfedge, setting its pm field to 1
-                # insert the new Halfedge to the right of the original bisector
-                lbnd = bisector
-                bisector = Halfedge(edge, Edge.RE)
-                edgeList.insert(lbnd, bisector)
+            # create a new Halfedge, setting its pm field to 1
+            # insert the new Halfedge to the right of the original bisector
+            lbnd = bisector
+            bisector = Halfedge(edge, Edge.RE)
+            edgeList.insert(lbnd, bisector)
 
-                # if this new bisector intersects with the right Halfedge
-                p = bisector.intersect(rbnd)
-                if p is not None:
-                    # push the Halfedge into the ordered linked list of vertices
-                    priorityQ.insert(bisector, p, newsite.distance(p))
+            # if this new bisector intersects with the right Halfedge
+            p = bisector.intersect(rbnd)
+            if p is not None:
+                # push the Halfedge into the ordered linked list of vertices
+                priorityQ.insert(bisector, p, newsite.distance(p))
 
-                newsite = next(siteIter)
+            newsite = next(siteIter)
 
-            elif not priorityQ.isEmpty():
-                # intersection is smallest - this is a vector (circle) event
+        elif not priorityQ.isEmpty():
+            # intersection is smallest - this is a vector (circle) event
 
-                # pop the Halfedge with the lowest vector off the ordered list of
-                # vectors.  Get the Halfedge to the left and right of the above HE
-                # and also the Halfedge to the right of the right HE
-                lbnd = priorityQ.popMinHalfedge()
-                llbnd = lbnd.left
-                rbnd = lbnd.right
-                rrbnd = rbnd.right
+            # pop the Halfedge with the lowest vector off the ordered list of
+            # vectors.  Get the Halfedge to the left and right of the above HE
+            # and also the Halfedge to the right of the right HE
+            lbnd = priorityQ.popMinHalfedge()
+            llbnd = lbnd.left
+            rbnd = lbnd.right
+            rrbnd = rbnd.right
 
-                # get the Site to the left of the left HE and to the right of
-                # the right HE which it bisects
-                bot = lbnd.leftreg(bottomsite)
-                top = rbnd.rightreg(bottomsite)
+            # get the Site to the left of the left HE and to the right of
+            # the right HE which it bisects
+            bot = lbnd.leftreg(bottomsite)
+            top = rbnd.rightreg(bottomsite)
 
-                # output the triple of sites, stating that a circle goes through them
-                mid = lbnd.rightreg(bottomsite)
-                context.outTriple(bot, top, mid)
+            # output the triple of sites, stating that a circle goes through them
+            mid = lbnd.rightreg(bottomsite)
+            context.outTriple(bot, top, mid)
 
-                # get the vertex that caused this event and set the vertex number
-                # couldn't do this earlier since we didn't know when it would be processed
-                v = lbnd.vertex
-                siteList.setSiteNumber(v)
-                context.outVertex(v)
+            # get the vertex that caused this event and set the vertex number
+            # couldn't do this earlier since we didn't know when it would be processed
+            v = lbnd.vertex
+            siteList.setSiteNumber(v)
+            context.outVertex(v)
 
-                # set the endpoint of the left and right Halfedge to be this vector
-                if lbnd.edge.setEndpoint(lbnd.pm, v):
-                    context.outEdge(lbnd.edge)
+            # set the endpoint of the left and right Halfedge to be this vector
+            if lbnd.edge.setEndpoint(lbnd.pm, v):
+                context.outEdge(lbnd.edge)
 
-                if rbnd.edge.setEndpoint(rbnd.pm, v):
-                    context.outEdge(rbnd.edge)
+            if rbnd.edge.setEndpoint(rbnd.pm, v):
+                context.outEdge(rbnd.edge)
 
-                # delete the lowest HE, remove all vertex events to do with the
-                # right HE and delete the right HE
-                edgeList.delete(lbnd)
-                priorityQ.delete(rbnd)
-                edgeList.delete(rbnd)
+            # delete the lowest HE, remove all vertex events to do with the
+            # right HE and delete the right HE
+            edgeList.delete(lbnd)
+            priorityQ.delete(rbnd)
+            edgeList.delete(rbnd)
 
-                # if the site to the left of the event is higher than the Site
-                # to the right of it, then swap them and set 'pm' to RIGHT
-                pm = Edge.LE
-                if bot.y > top.y:
-                    bot, top = top, bot
-                    pm = Edge.RE
+            # if the site to the left of the event is higher than the Site
+            # to the right of it, then swap them and set 'pm' to RIGHT
+            pm = Edge.LE
+            if bot.y > top.y:
+                bot, top = top, bot
+                pm = Edge.RE
 
-                # Create an Edge (or line) that is between the two Sites.  This
-                # creates the formula of the line, and assigns a line number to it
-                edge = Edge.bisect(bot, top)
-                context.outBisector(edge)
+            # Create an Edge (or line) that is between the two Sites.  This
+            # creates the formula of the line, and assigns a line number to it
+            edge = Edge.bisect(bot, top)
+            context.outBisector(edge)
 
-                # create a HE from the edge
-                bisector = Halfedge(edge, pm)
+            # create a HE from the edge
+            bisector = Halfedge(edge, pm)
 
-                # insert the new bisector to the right of the left HE
-                # set one endpoint to the new edge to be the vector point 'v'
-                # If the site to the left of this bisector is higher than the right
-                # Site, then this endpoint is put in position 0; otherwise in pos 1
-                edgeList.insert(llbnd, bisector)
-                if edge.setEndpoint(Edge.RE - pm, v):
-                    context.outEdge(edge)
+            # insert the new bisector to the right of the left HE
+            # set one endpoint to the new edge to be the vector point 'v'
+            # If the site to the left of this bisector is higher than the right
+            # Site, then this endpoint is put in position 0; otherwise in pos 1
+            edgeList.insert(llbnd, bisector)
+            if edge.setEndpoint(Edge.RE - pm, v):
+                context.outEdge(edge)
 
-                # if left HE and the new bisector don't intersect, then delete
-                # the left HE, and reinsert it
-                p = llbnd.intersect(bisector)
-                if p is not None:
-                    priorityQ.delete(llbnd)
-                    priorityQ.insert(llbnd, p, bot.distance(p))
+            # if left HE and the new bisector don't intersect, then delete
+            # the left HE, and reinsert it
+            p = llbnd.intersect(bisector)
+            if p is not None:
+                priorityQ.delete(llbnd)
+                priorityQ.insert(llbnd, p, bot.distance(p))
 
-                # if right HE and the new bisector don't intersect, then reinsert it
-                p = bisector.intersect(rrbnd)
-                if p is not None:
-                    priorityQ.insert(bisector, p, bot.distance(p))
-            else:
-                break
+            # if right HE and the new bisector don't intersect, then reinsert it
+            p = bisector.intersect(rrbnd)
+            if p is not None:
+                priorityQ.insert(bisector, p, bot.distance(p))
+        else:
+            break
+
+    he = edgeList.leftend.right
+    while he is not edgeList.rightend:
+        context.outEdge(he.edge)
+        he = he.right
+    Edge.EDGE_NUM = 0
 
-        he = edgeList.leftend.right
-        while he is not edgeList.rightend:
-            context.outEdge(he.edge)
-            he = he.right
-        Edge.EDGE_NUM = 0
-    except Exception as err:
-        # fix_print_with_import
-        print("######################################################")
-        # fix_print_with_import
-        print(str(err))
 
 # ------------------------------------------------------------------
 
@@ -373,6 +369,7 @@ def isEqual(a, b, relativeError=TOLERANCE):
     norm = max(abs(a), abs(b))
     return (norm < relativeError) or (abs(a - b) < (relativeError * norm))
 
+
 # ------------------------------------------------------------------
 
 
@@ -405,6 +402,7 @@ class Site(object):
         dy = self.y - other.y
         return math.sqrt(dx * dx + dy * dy)
 
+
 # ------------------------------------------------------------------
 
 
@@ -412,7 +410,7 @@ class Edge(object):
     LE = 0
     RE = 1
     EDGE_NUM = 0
-    DELETED = {}   # marker value
+    DELETED = {}  # marker value
 
     def __init__(self):
         self.a = 0.0
@@ -473,10 +471,10 @@ class Edge(object):
 class Halfedge(object):
 
     def __init__(self, edge=None, pm=Edge.LE):
-        self.left = None   # left Halfedge in the edge list
-        self.right = None   # right Halfedge in the edge list
-        self.qnext = None   # priority queue linked list pointer
-        self.edge = edge   # edge list Edge
+        self.left = None  # left Halfedge in the edge list
+        self.right = None  # right Halfedge in the edge list
+        self.qnext = None  # priority queue linked list pointer
+        self.edge = edge  # edge list Edge
         self.pm = pm
         self.vertex = None  # Site()
         self.ystar = BIG_FLOAT
@@ -537,13 +535,13 @@ class Halfedge(object):
         topsite = e.reg[1]
         right_of_site = pt.x > topsite.x
 
-        if(right_of_site and self.pm == Edge.LE):
+        if (right_of_site and self.pm == Edge.LE):
             return True
 
-        if(not right_of_site and self.pm == Edge.RE):
+        if (not right_of_site and self.pm == Edge.RE):
             return False
 
-        if(e.a == 1.0):
+        if (e.a == 1.0):
             dyp = pt.y - topsite.y
             dxp = pt.x - topsite.x
             fast = 0
@@ -552,14 +550,14 @@ class Halfedge(object):
                 fast = above
             else:
                 above = pt.x + pt.y * e.b > e.c
-                if(e.b < 0.0):
+                if (e.b < 0.0):
                     above = not above
                 if (not above):
                     fast = 1
             if (not fast):
                 dxs = topsite.x - (e.reg[0]).x
                 above = e.b * (dxp * dxp - dyp * dyp) < dxs * dyp * (1.0 + 2.0 * dxp / dxs + e.b * e.b)
-                if(e.b < 0.0):
+                if (e.b < 0.0):
                     above = not above
         else:  # e.b == 1.0
             yl = e.c - e.a * pt.x
@@ -568,7 +566,7 @@ class Halfedge(object):
             t3 = yl - topsite.y
             above = t1 * t1 > t2 * t2 + t3 * t3
 
-        if(self.pm == Edge.LE):
+        if (self.pm == Edge.LE):
             return above
         else:
             return not above
@@ -591,7 +589,7 @@ class Halfedge(object):
 
         xint = (e1.c * e2.b - e2.c * e1.b) / d
         yint = (e2.c * e1.a - e1.c * e2.a) / d
-        if(cmp(e1.reg[1], e2.reg[1]) < 0):
+        if (cmp(e1.reg[1], e2.reg[1]) < 0):
             he = self
             e = e1
         else:
@@ -599,8 +597,8 @@ class Halfedge(object):
             e = e2
 
         rightOfSite = xint >= e.reg[1].x
-        if((rightOfSite and he.pm == Edge.LE) or
-           (not rightOfSite and he.pm == Edge.RE)):
+        if ((rightOfSite and he.pm == Edge.LE)
+                or (not rightOfSite and he.pm == Edge.RE)):
             return None
 
         # create a new site at the point of intersection - this is a new
@@ -640,7 +638,7 @@ class EdgeList(object):
 
     # Get entry from hash table, pruning any deleted nodes
     def gethash(self, b):
-        if(b < 0 or b >= self.hashsize):
+        if (b < 0 or b >= self.hashsize):
             return None
         he = self.hash[b]
         if he is None or he.edge is not Edge.DELETED:
@@ -654,14 +652,14 @@ class EdgeList(object):
         # Use hash table to get close to desired halfedge
         bucket = int(((pt.x - self.xmin) / self.deltax * self.hashsize))
 
-        if(bucket < 0):
+        if (bucket < 0):
             bucket = 0
 
-        if(bucket >= self.hashsize):
+        if (bucket >= self.hashsize):
             bucket = self.hashsize - 1
 
         he = self.gethash(bucket)
-        if(he is None):
+        if (he is None):
             i = 1
             while True:
                 he = self.gethash(bucket - i)
@@ -672,7 +670,7 @@ class EdgeList(object):
                     break
                 i += 1
 
-        # Now search linear list of halfedges for the corect one
+        # Now search linear list of halfedges for the correct one
         if (he is self.leftend) or (he is not self.rightend and he.isPointRightOf(pt)):
             he = he.right
             while he is not self.rightend and he.isPointRightOf(pt):
@@ -684,7 +682,7 @@ class EdgeList(object):
                 he = he.left
 
         # Update hash table and reference counts
-        if(bucket > 0 and bucket < self.hashsize - 1):
+        if (bucket > 0 and bucket < self.hashsize - 1):
             self.hash[bucket] = he
         return he
 
@@ -713,7 +711,7 @@ class PriorityQueue(object):
         he.ystar = site.y + offset
         last = self.hash[self.getBucket(he)]
         next = last.qnext
-        while((next is not None) and cmp(he, next) > 0):
+        while ((next is not None) and cmp(he, next) > 0):
             last = next
             next = last.qnext
         he.qnext = last.qnext
@@ -740,7 +738,7 @@ class PriorityQueue(object):
         return bucket
 
     def getMinPt(self):
-        while(self.hash[self.minidx].qnext is None):
+        while (self.hash[self.minidx].qnext is None):
             self.minidx += 1
         he = self.hash[self.minidx].qnext
         x = he.vertex.x
@@ -841,6 +839,7 @@ def computeVoronoiDiagram(points):
     voronoi(siteList, context)
     return (context.vertices, context.lines, context.edges)
 
+
 # ------------------------------------------------------------------
 
 
